@@ -5,6 +5,11 @@ use warnings;
 
 use BEGIN::Lift;
 use Moonshine::Element;
+use Moonshine::Util "assert_valid_html5_tag" => { -as => "assert_valid_tag" };
+
+use MOP::Class;
+use MOP::Role;
+use Carp qw/croak/;
 
 =head1 NAME
 
@@ -12,11 +17,11 @@ Moonshine::Magic - d[ o_0 ]b - has, extends, lazy_components
 
 =head1 VERSION
 
-Version 0.05
+Version 0.06
 
 =cut
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 =head1 SYNOPSIS
 
@@ -38,6 +43,10 @@ our $VERSION = '0.05';
     # BEGIN { @HAS = ( 'modify_spec' => sub { { switch => 0, switch_base => 0 } } }    
     # $self->modify_spec
 
+=head2 with
+
+    with 'Moonshine::Component::Glyphicon';
+
 =head2 lazy_components
 
     lazy_components (qw/p span h1/);
@@ -49,50 +58,93 @@ sub import {
     my $caller = caller;
 
     BEGIN::Lift::install(
-        ($caller, 'extends') => sub {
+        ( $caller, 'extends' ) => sub {
             no strict 'refs';
-            @{"${caller}::ISA"} = @_;
+            my @extends = @_;
+
+            unshift @extends, 'UNIVERSAL::Object'
+              unless grep { $_ eq 'UNIVERSAL::Object' } @extends;
+
+            for (@extends) {
+                eval "require $_";
+                croak $@ if $@;
+            }
+
+            @{"${caller}::ISA"} = @extends;
         }
     );
 
     BEGIN::Lift::install(
-        ($caller, 'has') => sub {
+        ( $caller, 'has' ) => sub {
             my %args = @_;
             no strict 'refs';
             no warnings 'once';
             %{"${caller}::HAS"} = %args;
-            for my $arg (keys %args) {
-                *{"${caller}::${arg}"} = sub { return $args{$arg}->(); };
+
+            my $class = MOP::Class->new($caller);
+            for my $arg ( keys %args ) {
+                $class->add_method( $arg, sub { return $args{$arg}->(); } );
             }
         }
     );
 
     BEGIN::Lift::install(
-        ($caller, 'lazy_components') => sub {
-	        my @lazy_components = @_;
-            no strict 'refs';
-            no warnings 'once';
-            for my $component (@lazy_components) {
-		        *{"${caller}::${component}"} = sub {
-			        my $self = shift;
-                   
-                    my ($base_args, $build_args) = ();
-                    if ($self->can('validate_build')) {
-                        ( $base_args, $build_args ) = $self->validate_build({
-                            params => $_[0] // {},
-                            spec => {
-                                tag  => { default => $component },
-                                data => 0,
-                                ( $_[1] ? %{ $_[1] } : () )
-                            }
-                        });
-                    } else {
-                        $base_args = { tag => $component, %{ $_[0] } }; 
-                    }
+        ( $caller, 'with' ) => sub {
+            my @roles = @_;
+            croak "No roles supplied!" unless @roles;
+            my $class = MOP::Class->new($caller);
+            for my $r (@roles) {
+                eval "require $r";
+                croak $@ if $@;
 
-                    return Moonshine::Element->new($base_args);
+                my $role = MOP::Class->new($r);
+                for my $meth ( $role->all_methods ) {
+                    next if $meth->name eq '__ANON__';
+                    $class->add_method( $meth->name, $meth->body );
                 }
-	        }
+
+                for my $slot ( $role->all_slots ) {
+                    next if $class->has_slot($slot->name);
+                    $class->alias_slot( $slot->name, $slot->initializer );
+                    next if $class->has_method($slot->name);
+                    $class->add_method( $slot->name, $slot->initializer );
+                }
+            }
+        }
+    );
+
+    BEGIN::Lift::install(
+        ( $caller, 'lazy_components' ) => sub {
+            my @lazy_components = @_;
+            my $class           = MOP::Class->new($caller);
+            for my $component (@lazy_components) {
+                next unless assert_valid_tag($component);
+                $class->add_method(
+                    $component,
+                    sub {
+                        my $self = shift;
+
+                        my ( $base_args, $build_args ) = ();
+                        if ( $self->can('validate_build') ) {
+                            ( $base_args, $build_args ) = $self->validate_build(
+                                {
+                                    params => $_[0] // {},
+                                    spec => {
+                                        tag  => { default => $component },
+                                        data => 0,
+                                        ( $_[1] ? %{ $_[1] } : () )
+                                    }
+                                }
+                            );
+                        }
+                        else {
+                            $base_args = { tag => $component, %{ $_[0] } };
+                        }
+
+                        return Moonshine::Element->new($base_args);
+                    }
+                );
+            }
         }
     );
 }
@@ -182,4 +234,4 @@ EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 =cut
 
-1; # End of Moonshine::Magic
+1;    # End of Moonshine::Magic
